/*****************************************************
Chip type               : ATmega8
AVR Core Clock frequency: 8,000000 MHz
*****************************************************/

#include <mega8.h>
#include <delay.h>

#define A PORTD.2
#define B PORTD.3
#define C PORTD.5
#define D PORTD.6
#define E PORTD.7
#define F PORTD.1
#define G PORTD.0
#define DP PORTB.7
#define A1 PORTD.4
#define A2 PORTB.6
#define A3 PORTB.0

#define ADC_itn 16

unsigned int ADC0_tmp [ADC_itn] ;
unsigned int ADC1_tmp [ADC_itn] ;
char counter; 
char display_buff [3], d_point_position;


void ports_init ()
{
 DDRB = 0b11000001; PORTB = 0b00111110;  
 DDRC = 0b00000000; PORTC = 0b11111100;
 DDRD = 0b11111111; PORTD = 0b00000000;
}

void ADC_init ()
{
 MCUSR |= 1<<SE|1<<SM0; //adc noise reduction 
 ADCSR |= 1<<ADEN|1<<ADIE|1<<ADPS2|1<<ADPS1|1<<ADPS0; // div by 128
 ADMUX |= 1<<REFS1|1<<REFS0; //2,56 bg refs
}

void convert_data (char cnnl, unsigned int adc)
{
 unsigned int volts;
 char i;
 
if (!cnnl) {volts = (((long)adc*999))>>10; d_point_position = 2;}
else {volts = (((long)adc*250))>>10; d_point_position = 1;}
for (i=0; i<3; i++) {display_buff [i] = volts%10; volts /= 10;} 
}

void clear_display () 
{ A=1; B=1; C=1; D=1; E=1; F=1; G=1; DP=1; A1 = 0; A2 = 0; A3 = 0; }
 
void display_data ()
{

 switch (display_buff[counter])
 {
  case 0: A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 1: A=1; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 2: A=0; B=0; C=1; D=0; E=0; F=1; G=0; break;
  case 3: A=0; B=0; C=0; D=0; E=1; F=1; G=0; break;
  case 4: A=1; B=0; C=0; D=1; E=1; F=0; G=0; break;
  case 5: A=0; B=1; C=0; D=0; E=1; F=0; G=0; break;
  case 6: A=0; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 7: A=0; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 8: A=0; B=0; C=0; D=0; E=0; F=0; G=0; break;
  case 9: A=0; B=0; C=0; D=0; E=1; F=0; G=0; break;
  default : A=1; B=1; C=1; D=1; E=1; F=1; G=0; break;  
   }
 
 switch (counter)
 {
  case 0: A1 = 1; break;
  case 1: A2 = 1; if(d_point_position == 1) {DP = 0;} break;
  case 2: A3 = 1; if(d_point_position == 2) {DP = 0;} break;
 }
  
}


interrupt [ADC_INT] void adc_isr(void)
{
 static bit ADC_cnl; 
 static char i = 0;
 static unsigned int ADC0_fr, ADC1_fr, counter2; 

if (!ADC_cnl)
 {
  for (i=0; i<(ADC_itn-1); i++ ) {ADC0_tmp [i] = ADC0_tmp [i+1];}
  ADC0_tmp [(ADC_itn-1)] = ADCW;
  ADC0_fr = 0;
  for (i=0; i<(ADC_itn); i++ ) { ADC0_fr += ADC0_tmp [i];}
  ADC0_fr /= ADC_itn;
 }
 else 
 {
  for (i=0; i<(ADC_itn-1); i++ ) {ADC1_tmp [i] = ADC1_tmp [i+1];}
  ADC1_tmp [(ADC_itn-1)] = ADCW;
  ADC1_fr = 0;
  for (i=0; i<(ADC_itn); i++ ) { ADC1_fr += ADC1_tmp [i];}
  ADC1_fr /= ADC_itn;
 }
 
 if (ADC0_fr < 1020) 
 {
  if (counter2) {counter2--;}  else { counter2 = 1000; convert_data (0, ADC0_fr);}
  ADC_cnl = 0; ADMUX &= 0b11111110;
  }
 else 
 {
  if (counter2) {counter2--;}  else { counter2 = 1000;  convert_data (1, ADC1_fr);}
  if (ADC_cnl) {ADC_cnl = 0; ADMUX &= 0b11111110;}
  else {ADC_cnl = 1; ADMUX |= 0b00000001;}
 }
 
  clear_display ();
  display_data ();
  if (counter < 2) {counter++;} else {counter = 0;}

#asm("wdr")
ADCSR |= 1<<ADSC;
}




void main(void)
{
 ports_init ();
 ADC_init (); 
 
#pragma optsize-
WDTCR=0x1E;
WDTCR=0x0E;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

#asm("sei")

ADCSR |= 1<<ADSC;

while (1) {#asm("sleep")}
}
